package collision;

import java.util.List;

public class Collider{

    public final CollisionDetector cd;
    int radius;
    int x, y;

    int iMin, iMax, jMin, jMax;
    boolean onBoard = false;
    boolean marked; //Used by the collision detector
    
    public Collider(CollisionDetector cd, int radius){
        this.cd = cd;
        this.radius = radius;
    }

    public void setSize(int radius){
        if(onBoard){
            throw new RuntimeException("Cannot change size while onboard; remove first");
        }
        this.radius = radius;
    }
    
    public boolean register(){
        if(onBoard){
            throw new RuntimeException("Already on board!");
        }
        if(!checkBounds(x, y)){
            return false;
        }
        for(int i = iMin; i <= iMax; i++){
            for(int j = jMin; j <= jMax; j++){
                Tile tile = cd.tiles[i + cd.width * j];
                tile.contents.add(this);
            }
        }
        onBoard = true;
        return true;
    }
    
    public void remove(){
        if(!onBoard){
            throw new RuntimeException("Already off board!");
        }
        for(int i = iMin; i <= iMax; i++){
            for(int j = jMin; j <= jMax; j++){
                cd.tiles[i + cd.width * j].contents.remove(this);
            }
        }
        onBoard = false;
    }
    
    public void set(int x, int y){
        if(onBoard){
            throw new RuntimeException("Cannot set position of collider while onboard; use move method instead");
        }
        iMin = (x - radius)/cd.resolution;
        jMin = (y - radius)/cd.resolution;
        iMax = (x + radius)/cd.resolution;
        jMax = (y + radius)/cd.resolution;
        this.x = x;
        this.y = y;
    }
    
    public boolean move(int x, int y){
        if(!onBoard){
            throw new RuntimeException("Cannot move collider while offboard; use set method instead");
        }
        int iMin = (x - radius)/cd.resolution;
        int jMin = (y - radius)/cd.resolution;
        int iMax = (x + radius)/cd.resolution;
        int jMax = (y + radius)/cd.resolution;
        if(iMin != this.iMin || jMin != this.jMin || iMax != this.iMax || jMax != this.jMax){
            remove();
            set(x, y);
            return register();
        } else {
            this.x = x;
            this.y = y;
            return true;
        }
    }
    
    public boolean checkCollide(){
        return checkCollide(x, y);
    }
    
    public boolean checkCollide(int x, int y){
        int dx, dy;
        float dSquared, radiusSum;
        Collider other;
        
        int iMin = (x - radius)/cd.resolution;
        int jMin = (y - radius)/cd.resolution;
        int iMax = (x + radius)/cd.resolution;
        int jMax = (y + radius)/cd.resolution;
        
        if(!checkBounds(x, y)){
            return true; //"collides" with edge of world
        }
                
        for(int i = iMin; i <= iMax; i++){
            for(int j = jMin; j <= jMax; j++){
                List<Collider> colliders = cd.tiles[i + cd.width * j].contents;
                int len = colliders.size();
                for(int k = 0; k < len; k++){
                    other = colliders.get(k);
                    dx = this.x - other.x;
                    dy = this.y - other.y;
                    dSquared = (float)dx*dx + dy*dy;
                    radiusSum = this.radius + other.radius;
                    if(dSquared < radiusSum*radiusSum && other != this){
                        return true;
                    }
                }
            }
        }
        return false;
    }
    
    public boolean checkBounds(int x, int y){
        return (x > radius) && (x + radius < cd.xPoints) && (y > radius) && (y + radius < cd.yPoints);
    }
    
    public boolean isOnBoard(){
        return onBoard;
    }

    public int getX(){
        return x;
    }
    
    public int getY(){
        return y;
    }
    
    public int getRadius(){
        return radius;
    }
    
}
