package collision;

import java.util.List;
import java.util.ArrayList;

public class CollisionDetector{

  final Tile[] tiles;

  public final int width, height, resolution;
  public final int xPoints, yPoints;
  
  /*
   * Creates a collision detector with width times height square tiles where each tile is resolution times 
   * resolution units
   */
  public CollisionDetector(int width, int height, int resolution){
      this.width = width;
      this.height = height;
      this.resolution = resolution;
      xPoints = width*resolution;
      yPoints = height*resolution;
      tiles = new Tile[width*height];
      for(int i = 0; i < tiles.length; i++){
          tiles[i] = new Tile();
      }
  }
  
  /*
   * Returns a collider which has the specified radius and is associated with this collision detector
   */
  public Collider makeCollider(int radius){
      return new Collider(this, radius);
  }

  /*
   * Returns a List of all the colliders that intersect the specified circle
   */
  public List<Collider> getOverlapping(int x, int y, int radius){
      List<Collider> dst = new ArrayList<Collider>();
      getOverlapping(x, y, radius, dst);
      return dst;
  }
  
  /*
   * Loads all colliders that intersect the specified circle into the list dst
   */
  public void getOverlapping(int x, int y, int radius, List<Collider> dst){
      ArrayList<Collider> list = new ArrayList<Collider>();
      int iMin = (x - radius)/resolution;
      int jMin = (y - radius)/resolution;
      int iMax = (x + radius)/resolution;
      int jMax = (y + radius)/resolution;

      for(int i = iMin; i <= iMax; i++){
          for(int j = jMin; j < jMax; j++){
              Tile tile = tiles[i + width*j];
              Collider c;
              int size = tile.contents.size();
              for(int k = 0; k < size; k++){
                  c = tile.contents.get(k);
                  if(c.marked){
                      continue; //already added this to the list
                  }
                  if(check(c, x, y, radius)){
                      dst.add(c);
                      c.marked = true;
                  }
              }
          }
      }
      int size = list.size();
      for(int i = 0; i < size; i++){
          list.get(i).marked = false;
      }
  }
    
  /*
   * Checks whether any collider intersects the specified circle. Returns the first collider found if any, otherwise
   * null.
   */
  public Collider check(int x, int y, int radius){
      int iMin = (x - radius)/resolution;
      int jMin = (y - radius)/resolution;
      int iMax = (x + radius)/resolution;
      int jMax = (y + radius)/resolution;
      
      for(int i = iMin; i <= iMax; i++){
          for(int j = jMin; j <= jMax; j++){
              Tile tile = tiles[i + width*j];
              int size = tile.contents.size();
              Collider c;
              for(int k = 0; k < size; k++){
                  c = tile.contents.get(k);
                  if(check(c, x, y, radius)){
                      return c;
                  }
              }
          }
      }
      return null;
  }
  
  /**
   * Checks whether the given collider intersects the specified circle
   */
  private boolean check(Collider c, int x, int y, int radius){
      float dx = c.x - x; //use floats to avoid overflow
      float dy = c.y - y;
      float radiusSum = c.radius + radius;
      return dx*dx + dy*dy < radiusSum * radiusSum;
  }
  
  public Tile getTile(int i, int j){
      return tiles[i + width*j];
  }
  
}
